#version 330

layout(points) in;
layout(points) out;
layout(max_vertices = 30) out;

in float Type0[];
in vec3 Position0[];
in vec3 Velocity0[];
in float Age0[];

out float Type1;
out vec3 Position1;
out vec3 Velocity1;
out float Age1;

uniform float gDeltaTimeMillis;
uniform float gTime;
uniform sampler1D gRandomTexture;
uniform float gLauncherLifetime;
uniform float gShellLifetime;
uniform float gSecondaryShellLifetime;
uniform float gParticleLifetime;
uniform vec3 gDirection;

#define PARTICLE_TYPE_LAUNCHER 0.0f
#define PARTICLE_TYPE_SHELL 1.0f
#define PARTICLE_TYPE_SECONDARY_SHELL 2.0f

#define DIRECTION_Z_POSITIVE 1
#define DIRECTION_Z_NEGATIVE 2
#define DIRECTION_Y_POSITIVE 3
#define DIRECTION_Y_NEGATIVE 4
#define DIRECTION_X_POSITIVE 5
#define DIRECTION_X_NEGATIVE 6
#define DIRECTION_UNKWON 7

vec3 getRandomDir(float TexCoord) {
	vec3 dir = texture(gRandomTexture, TexCoord).xyz;
	dir -= vec3(0.5, 0.5, 0.5);
	return dir;
}

void main()
{
	float Age = Age0[0] + gDeltaTimeMillis;

	if (Type0[0] == PARTICLE_TYPE_LAUNCHER) {

		if (Age >= gLauncherLifetime) {
			Type1 = PARTICLE_TYPE_SHELL;
			Position1 = Position0[0];
			Age1 = 0.0;

			vec3 Dir = vec3(0.0f, 0.0f, 0.0f);

			int directionCase = 0;

			if(gDirection.x == 0 && gDirection.y == 0) {
				directionCase = (gDirection.z > 0) ? DIRECTION_Z_POSITIVE : DIRECTION_Z_NEGATIVE;
			} else if(gDirection.x == 0 && gDirection.z == 0) {
				directionCase = (gDirection.y > 0) ? DIRECTION_Y_POSITIVE : DIRECTION_Y_NEGATIVE;
			} else if(gDirection.y == 0 && gDirection.z == 0) {
				directionCase = (gDirection.x > 0) ? DIRECTION_X_POSITIVE : DIRECTION_X_NEGATIVE;
			} else
				directionCase = DIRECTION_UNKWON;
			// calculate the direction

			for(int i = 0; i < 360; i += 20) {
						
				switch(directionCase) {
					case DIRECTION_Z_POSITIVE:
						Dir = vec3(sin(i), cos(i), -0.5f);
						break;
					case DIRECTION_Z_NEGATIVE:
						Dir = vec3(sin(i), cos(i), 0.5f);
						break;
					case DIRECTION_Y_POSITIVE:
						Dir = vec3(sin(i), -0.5f , cos(i));
						break;
					case DIRECTION_Y_NEGATIVE:
						Dir = vec3(sin(i), 0.5f , cos(i));
						break;
					case DIRECTION_X_POSITIVE:
						Dir = vec3(-0.5f, sin(i) , cos(i));
						break;
					case DIRECTION_X_NEGATIVE:
						Dir = vec3(0.5f, sin(i) , cos(i));
						break;
					default:
						Dir = normalize(getRandomDir(gTime / 1000.0f));
						break;
				}

				Velocity1 = normalize(Dir) / 20.0f;
			
				EmitVertex();
				EndPrimitive();
			}
			
			Age = 0.0;
		}
		
		//if (Age >= gLauncherLifetime) {
		//	Type1 = PARTICLE_TYPE_SHELL;
		//	Position1 = Position0[0];
			
		//	Dir = gDirection * Dir;
		//	//Dir.y = max(Dir.y, 0.5);
		//	Velocity1 = normalize(Dir) / 20.0;
		//	Age1 = 0.0;
		//	EmitVertex();
		//	EndPrimitive();
				
		//	Age = 0.0;
		//} 

		if(gTime < gParticleLifetime) {
			Type1 = PARTICLE_TYPE_LAUNCHER;
			Position1 = Position0[0];
			Velocity1 = Velocity0[0];
			Age1 = Age;
			EmitVertex();
			EndPrimitive();
		}
		
	} else {

		float DeltaTimeSecs = gDeltaTimeMillis / 1000.0f;
		vec3 DeltaP = DeltaTimeSecs * Velocity0[0] * 30;
		vec3 DeltaV = vec3(DeltaTimeSecs) * (0.0, -9.81, 0.0);

		if (Type0[0] == PARTICLE_TYPE_SHELL)  {

			if (Age < gShellLifetime) {
				Type1 = PARTICLE_TYPE_SHELL;
				Position1 = Position0[0] + DeltaP;
				Velocity1 = Velocity0[0] + DeltaV;
				Age1 = Age;
				EmitVertex();
				EndPrimitive();
			} else {
				for (int i = 0 ; i < 10 ; i++) {
					Type1 = PARTICLE_TYPE_SECONDARY_SHELL;
					Position1 = Position0[0];
					vec3 Dir = getRandomDir((gTime + i)/1000.0);
					Velocity1 = normalize(Dir) / 20.0;
					Age1 = 0.0f;
					EmitVertex();
					EndPrimitive();
				}
			}
		} else {

			if (Age < gSecondaryShellLifetime) {
				Type1 = PARTICLE_TYPE_SECONDARY_SHELL;
				Position1 = Position0[0] + DeltaP;
				Velocity1 = Velocity0[0] + DeltaV;
				Age1 = Age;
				EmitVertex();
				EndPrimitive();
			} else {
				EndPrimitive();
			}
		}
	}
}